@@ -207,6 +207,7 @@ class BoardConfig(object):
     mmc_part_offset = 0
     uimage_path = ''
     fat_size = 32
+    extra_serial_opts = ''
     _extra_serial_opts = ''
     _live_serial_opts = ''
     extra_boot_args_options = None
@@ -573,6 +574,20 @@ class BoardConfig(object):
                 "bootm %(kernel_addr)s %(initrd_addr)s" % replacements)
         return boot_script
 
+    @classmethod
+    def add_boot_args(cls, extra_args):
+        if extra_args is not None:
+            if cls.extra_boot_args_options is None:
+                cls.extra_boot_args_options = extra_args
+            else:
+                cls.extra_boot_args_options += ' %s' % extra_args
+
+    @classmethod
+    def add_boot_args_from_file(cls, path):
+        if path is not None:
+            with open(path, 'r') as boot_args_file:
+                cls.add_boot_args(boot_args_file.read().strip())
+
     @classmethod
     def _get_bootargs(cls, is_live, is_lowmem, consoles, rootfs_uuid):
         """Get the bootargs for this board.
@@ -696,7 +711,7 @@ class BoardConfig(object):
 
         if (cls.snowball_startup_files_config is not None and
             cls.board != 'snowball_sd'):
-            cls.populate_raw_partition(chroot_dir, boot_device_or_file)
+            cls.populate_raw_partition(boot_device_or_file, chroot_dir)
 
         if cls.env_dd:
             # Do we need to zero out the env before flashing it?
@@ -805,6 +820,13 @@ class BoardConfig(object):
         # Override in subclass if needed
         pass
 
+    @classmethod
+    def snowball_config(cls, chroot_dir):
+        # Override in subclasses where applicable
+        raise NotImplementedError(
+            "snowball_config() must only be called on BoardConfigs that use the "
+            "Snowball startupfiles.")
+
 
 class OmapConfig(BoardConfig):
     kernel_flavors = ['linaro-omap4', 'linaro-lt-omap', 'linaro-omap', 'omap4']
@@ -1055,26 +1077,41 @@ class SnowballEmmcConfig(SnowballSdConfig):
         make_uImage(cls.load_addr, k_img_data, boot_dir)
         boot_script_path = os.path.join(boot_dir, cls.boot_script)
         make_boot_script(boot_env, boot_script_path)
-        cls.populate_raw_partition(chroot_dir, boot_device_or_file)
+        cls.populate_raw_partition(boot_device_or_file, chroot_dir)
 
     @classmethod
-    def populate_raw_partition(cls, chroot_dir, boot_device_or_file):
+    def populate_raw_partition(cls, boot_device_or_file, chroot_dir):
         # Populate created raw partition with TOC and startup files.
-        config_files_path = os.path.join(chroot_dir, 'boot')
         _, toc_filename = tempfile.mkstemp()
-        new_files = cls.get_file_info(chroot_dir)
+        config_files_dir = cls.snowball_config(chroot_dir)
+        new_files = cls.get_file_info(chroot_dir, config_files_dir)
         with open(toc_filename, 'wb') as toc:
             cls.create_toc(toc, new_files)
         cls.install_snowball_boot_loader(toc_filename, new_files,
-                                     boot_device_or_file,
-                                     cls.SNOWBALL_LOADER_START_S)
+                                         boot_device_or_file,
+                                         cls.SNOWBALL_LOADER_START_S,
+                                         cls.delete_startupfiles)
         cls.delete_file(toc_filename)
-        cls.delete_file(os.path.join(config_files_path,
-                                     cls.snowball_startup_files_config))
+        if cls.delete_startupfiles:
+            cls.delete_file(os.path.join(config_files_dir,
+                                         cls.snowball_startup_files_config))
+
+    @classmethod
+    def snowball_config(cls, chroot_dir):        
+        # We will find the startupfiles in the target boot partition.
+        return os.path.join(chroot_dir, 'boot')
+
+    @classproperty
+    def delete_startupfiles(cls):
+        # The startupfiles will have been installed to the target boot
+        # partition by the hwpack, and should be deleted so we don't leave
+        # them on the target system.
+        return True
 
     @classmethod
     def install_snowball_boot_loader(cls, toc_file_name, files,
-                                     boot_device_or_file, start_sector):
+                                     boot_device_or_file, start_sector,
+                                     delete_startupfiles=False):
         ''' Copies TOC and boot files into the boot partition.
         A sector size of 1 is used for some files, as they do not
         necessarily start on an even address. '''
@@ -1092,7 +1129,8 @@ class SnowballEmmcConfig(SnowballSdConfig):
             else:
                 seek_sectors = start_sector + file['offset'] / SECTOR_SIZE
                 _dd(filename, boot_device_or_file, seek=seek_sectors)
-            cls.delete_file(filename)
+            if delete_startupfiles:
+                cls.delete_file(filename)
 
     @classmethod
     def delete_file(cls, file_path):
@@ -1123,14 +1161,14 @@ class SnowballEmmcConfig(SnowballSdConfig):
             f.write(data)
 
     @classmethod
-    def get_file_info(cls, chroot_dir):
+    def get_file_info(cls, chroot_dir, config_files_dir):
         ''' Fills in the offsets of files that are located in
         non-absolute memory locations depending on their sizes.'
         Also fills in file sizes'''
         ofs = cls.TOC_SIZE
         files = []
-        bin_dir = os.path.join(chroot_dir, 'boot')
-        with open(os.path.join(bin_dir, cls.snowball_startup_files_config),
+        with open(os.path.join(config_files_dir,
+                               cls.snowball_startup_files_config),
                   'r') as info_file:
             for line in info_file:
                 file_data = line.split()
@@ -1143,7 +1181,7 @@ class SnowballEmmcConfig(SnowballSdConfig):
                     filename = os.path.join(chroot_dir,
                                             file_data[1].lstrip('/'))
                 else:
-                    filename = os.path.join(bin_dir, file_data[1])
+                    filename = os.path.join(config_files_dir, file_data[1])
                 assert os.path.exists(filename), "File %s does not exist, " \
                     "please check the startfiles config file." % file_data[1]
                 address = long(file_data[3], 16)
@@ -1383,7 +1421,7 @@ class SamsungConfig(BoardConfig):
         return uboot_file
 
     @classmethod
-    def populate_raw_partition(cls, chroot_dir, boot_device_or_file):
+    def populate_raw_partition(cls, boot_device_or_file, chroot_dir):
         # Zero the env so that the boot_script will get loaded
         _dd("/dev/zero", boot_device_or_file, count=cls.SAMSUNG_V310_ENV_LEN,
             seek=cls.SAMSUNG_V310_ENV_START)
